# - Open Porous Media Initiative shared infrastructure config mode
#
# Defines the following variables:
#  opm-common_FOUND        - true
#  opm-common_VERSION      - version of the opm-common library found, e.g. 0.2
#  opm-common_DEFINITIONS  - defines to be made on the command line
#  opm-common_INCLUDE_DIRS - header directories with which to compile
#  opm-common_LINKER_FLAGS - flags that must be passed to the linker
#  opm-common_LIBRARIES    - names of the libraries with which to link
#  opm-common_LIBRARY_DIRS - directories in which the libraries are situated
#
# You should put lines like this in your CMakeLists.txt
#  set (opm-common_DIR "${PROJECT_BINARY_DIR}/../opm-common" CACHE LOCATION "Build tree of opm-common")
#  find_package (opm-common)
#  configure_vars (
#    FILE  CXX  "${PROJECT_BINARY_DIR}/config.h"
#    WRITE ${opm-common_CONFIG_VARS}
#    )

# <http://www.vtk.org/Wiki/CMake/Tutorials/How_to_create_a_ProjectConfig.cmake_file>

# Prevent multiple inclusions
if(NOT opm-common_FOUND)
  # propagate these properties from one build system to the other
  set (opm-common_PREFIX "/usr")
  set (opm-common_VERSION "2021.10")
  set (opm-common_DEFINITIONS "")
  set (opm-common_INCLUDE_DIRS "/usr/include;/usr/include/cjson")
  set (opm-common_LIBRARY_DIRS "/usr/lib/mipsel-linux-gnu" "/usr/lib/mipsel-linux-gnu")
  set (opm-common_LINKER_FLAGS "")
  set (opm-common_CONFIG_VARS "HAVE_OPENMP")

  # The flag whether embedded Python is supported or not is passed to downstream modules,
  # it is only used to enable/disable regression testing with Python enabled input. The
  # actual code is self contained - and can be used downstream without awareness of this.
  set (opm-common_EMBEDDED_PYTHON 1)

  # libraries come from the build tree where this file was generated
  set (opm-common_LIBRARY "/usr/lib/mipsel-linux-gnu/libopmcommon.so")
  set (opm-common_LIBRARIES ${opm-common_LIBRARY} "fmt::fmt;/usr/lib/mipsel-linux-gnu/libpython3.10.so;OpenMP::OpenMP_CXX;Boost::system;/usr/lib/mipsel-linux-gnu/libcjson.so;/usr/lib/mipsel-linux-gnu/libm.so")

  # The purpose of this string replacement operation is to enable use of the
  # generated opm-project-config.cmake file also in the situation where 'make
  # install' has been invoked with the DESTDIR option:
  #
  # opm-common/build> cmake .. -DCMAKE_INSTALL_PREFIX=/real/prefix
  # opm-common/budil> make install DESTDIR=/tmp/prefix
  #
  # downstream/build> cmake .. -DDEST_PREFIX=/tmp/prefix -DCMAKE_PREFIX_PATH=/tmp/prefix
  # downstream/build> make install
  #
  # That way the downstream dependency can still use find_package( opm-common )
  # even though the opm-common-config.cmake file is not internally consistent
  # with it's own location in the filesystem.

  if(DEST_PREFIX)
    set(DEST_PREFIX "${DEST_PREFIX}${opm-common_PREFIX}")
    string(REPLACE ${opm-common_PREFIX} ${DEST_PREFIX} opm-common_INCLUDE_DIRS "${opm-common_INCLUDE_DIRS}")
    string(REPLACE ${opm-common_PREFIX} ${DEST_PREFIX} opm-common_LIBRARY_DIRS "${opm-common_LIBRARY_DIRS}")
    string(REPLACE ${opm-common_PREFIX} ${DEST_PREFIX} opm-common_LIBRARY      "${opm-common_LIBRARY}")
  endif()


  set (HAVE_OPM_COMMON 1)
  mark_as_advanced (opm-common_LIBRARY)

  # not all projects have targets; conditionally add this part
  if (NOT "opmcommon" STREQUAL "")
    # add the library as a target, so that other things in the project including
    # this file may depend on it and get rebuild if this library changes.
    if(NOT TARGET opmcommon)
      get_filename_component(_dir "${CMAKE_CURRENT_LIST_FILE}" PATH)
      include("${_dir}/opm-common-targets.cmake")
    endif()
  endif (NOT "opmcommon" STREQUAL "")

  # same as above, but for C99
  string (REPLACE "-std=c99" "" CMAKE_C_FLAGS "${CMAKE_C_FLAGS}")
  string (STRIP "${CMAKE_C_FLAGS}" CMAKE_C_FLAGS)
  set (CMAKE_C_FLAG "-std=c99 ${CMAKE_C_FLAGS}")

  # build with OpenMP if that was found
  if (NOT "-fopenmp" STREQUAL "")
    string (REPLACE "-fopenmp" "" CMAKE_C_FLAGS "${CMAKE_C_FLAGS}")
    string (STRIP "${CMAKE_C_FLAGS}" CMAKE_C_FLAGS)
    set (CMAKE_C_FLAG "-fopenmp ${CMAKE_C_FLAGS}")
  endif (NOT "-fopenmp" STREQUAL "")
  if (NOT "-fopenmp" STREQUAL "")
    string (REPLACE "-fopenmp" "" CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}")
    string (STRIP "${CMAKE_CXX_FLAGS}" CMAKE_CXX_FLAGS)
    set (CMAKE_C_FLAG "-fopenmp ${CMAKE_CXX_FLAGS}")
  endif (NOT "-fopenmp" STREQUAL "")

  # this is the contents of config.h as far as our probes can tell:

  # Require correct CMake standard. Needed for user modules as
  # some software will add incompatible compile switches like
  # -std=gnu++11 otherwise when search for (I guess because of
  # imported targets using INTERFACE_COMPILE_FEATURES), and will
  # break compilation because of missing c++17 features.
  if(NOT CMAKE_CXX_STANDARD)
    set(CMAKE_CXX_STANDARD 17)
    set(CMAKE_CXX_STANDARD_REQUIRED ON)
    set(CMAKE_CXX_EXTENSIONS OFF)
  endif()

  # The settings in this block do not mix well with the DEST_PREFIX
  # setting.
  if (NOT DEST_PREFIX)
    # This is required to include OpmPackage
    set(CMAKE_MODULE_PATH "${CMAKE_MODULE_PATH}" /usr/share/opm/cmake/Modules)

    # extra code from variable OPM_PROJECT_EXTRA_CODE
    #ENABLE_ECL_INPUT is needed by opm-common-prereq.cmake
                                       set(ENABLE_ECL_INPUT ON)
                                       set(OPM_MACROS_ROOT /usr/share/opm)
                                       list(APPEND CMAKE_MODULE_PATH ${OPM_MACROS_ROOT}/cmake/Modules)
                                       include(OpmPackage) #Make macros availabe after find_package(opm-common)
                                        set(COMPARE_ECL_COMMAND /usr/bin/compareECL)
                                        set(OPM_PACK_COMMAND /usr/bin/opmpack)
                                        set(RST_DECK_COMMAND /usr/bin/rst_deck)
                                            set(HAVE_ECL_INPUT 1)
                                            set(HAVE_ECL_OUTPUT 1)
    # end extra code

    include(OpmPackage)
    include(opm-common-prereqs)
  endif()
endif()
if (DEFINED HAVE_OPENMP AND NOT (("${HAVE_OPENMP}" STREQUAL "") OR ("${HAVE_OPENMP}" STREQUAL "1")))
	message (WARNING "Incompatible value \"${HAVE_OPENMP}\" of variable \"HAVE_OPENMP\"")
endif ()
set (HAVE_OPENMP "1")
